/* 
 * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

#import "WBSQLQueryPanel.h"
#import "MResultsetViewer.h"
#import "GRTIconCache.h"
#import "MVerticalLayoutView.h"
#import "WBTabView.h"
#import "WBSplitView.h"
#import "MCPPUtilities.h"
#import "MContainerView.h"
#import "WBSplitViewUnbrokenizerDelegate.h"
#import "WBOverviewPanel.h"
#import "ScintillaView.h"
#import "WBToolbarManager.h"////
#import "WBMiniToolbar.h"
#import "GRTListDataSource.h"

#import "MSQLEditorController.h"

#import "WBQuerySidebarController.h"

#include "sqlide/wb_sql_editor_snippets.h"

@interface ColorBallTextCell : NSTextFieldCell
{
}
@end

@implementation ColorBallTextCell

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView 
{
  [[self backgroundColor] set];
  
  NSAttributedString *text = [self attributedStringValue];
  NSSize textSize = [text size];
  
  cellFrame.origin.y += (cellFrame.size.height - cellFrame.size.width) / 2;
  cellFrame.size.height = cellFrame.size.width;
  
  cellFrame = NSInsetRect(cellFrame, 2, 2);
  
  [[NSBezierPath bezierPathWithOvalInRect:cellFrame] fill];
  
  cellFrame.origin.y += (cellFrame.size.height - textSize.height) / 2;
  [text drawInRect:cellFrame];
}

@end





@implementation WBSQLQueryPanel

#pragma mark Table View support

- (NSInteger) numberOfRowsInTableView: (NSTableView*) tableView;
{
  if (tableView == mMessagesTable)
  {
    if (mBackEnd)
      return mBackEnd->log()->count();
  }
  else if (tableView == mHistoryTable)
  {
    if (mBackEnd)
      return mBackEnd->history()->entries_model()->count();
  }
  else if (tableView == mHistoryDetailsTable)
  {
    if (mBackEnd && [mHistoryTable selectedRow] >= 0)
      return mBackEnd->history()->details_model()->count();
  }

  return 0;
}



- (id) tableView: (NSTableView*) aTableView
objectValueForTableColumn: (NSTableColumn*) aTableColumn
             row: (NSInteger) rowIndex;
{
  if (aTableView == mMessagesTable)
  {
    std::string text;
    
    if ([[aTableColumn identifier] isEqual:@"0"])
    {
      bec::IconId icon_id= mBackEnd->log()->get_field_icon(rowIndex, 0, bec::Icon16);
      
      if (icon_id != 0)
        return [[GRTIconCache sharedIconCache] imageForIconId:icon_id];
      else
        return nil;
    }
    else
    {
      mBackEnd->log()->get_field(rowIndex, [[aTableColumn identifier] intValue], text);
      
      return [NSString stringWithUTF8String: text.c_str()];
    }
  }
  else if (aTableView == mHistoryTable)
  {
    std::string text;
    
    mBackEnd->history()->entries_model()->get_field(rowIndex, [[aTableColumn identifier] intValue], text);
    
    return [NSString stringWithUTF8String: text.c_str()];
  }
  else if (aTableView == mHistoryDetailsTable)
  {
    std::string text;
    
    mBackEnd->history()->details_model()->get_field(rowIndex, [[aTableColumn identifier] intValue], text);
    
    return [NSString stringWithUTF8String: text.c_str()];
  }

  return @"foo";
}


- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
  NSTableView *sender= [aNotification object];
  
  if (sender == mHistoryTable)
  {
    if ([mHistoryTable selectedRow] >= 0)
      mBackEnd->history()->current_entry([mHistoryTable selectedRow]);
    [mHistoryDetailsTable reloadData];
  }
  else if (sender == mHistoryDetailsTable)
  {
  }
  else if (sender == mMessagesTable)
  {
    std::string text;
    mBackEnd->log()->get_field([mMessagesTable selectedRow], 3, text);
    [mOutputActionText setString: [NSString stringWithCPPString: text]];

    mBackEnd->log()->get_field([mMessagesTable selectedRow], 4, text);
    [mOutputResponseText setString: [NSString stringWithCPPString: text]];
  }
  else if (sender == snippetTable)
  {
    int row = [snippetTable selectedRow];
    if (row < 0)
      [mSnippetText setString: @""];
    else
    {
      std::string text;
      DbSqlEditorSnippets::get_instance()->get_field(row, DbSqlEditorSnippets::Script, text);
      [mSnippetText setString: [NSString stringWithCPPString: text]];
    }
  }  
}


- (CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)row
{
  if (tableView == mHistoryDetailsTable)
  {
    std::string text;
    mBackEnd->history()->details_model()->get_field(row, 0, text);
    size_t lines= 0;
    const char *ptr= text.c_str();
    
    do
    {
      ptr= strchr(ptr, '\n');
      lines++;
    } while (ptr++);

    return [tableView rowHeight] * lines;
  }
  else
    return [tableView rowHeight];
}


- (void)activateHistoryEntry:(id)sender
{
  
}


- (void)activateHistoryDetailEntry:(id)sender
{
  std::string text, query;
  NSIndexSet *selection= [mHistoryDetailsTable selectedRowIndexes];
  
  for (NSUInteger row= [selection firstIndex]; row != NSNotFound; row= [selection indexGreaterThanIndex:row])
  {
    mBackEnd->history()->details_model()->get_field(row, 0, text);
    query.append("\n").append(text);
  }

  MSQLEditorController *editor = [self activeEditor];
  
  [editor setString: [[editor string] stringByAppendingString: [NSString stringWithCPPString: query]]];
}


- (void)displayResultsets
{
  int n, rs_count;

  for (n = 0, rs_count = mBackEnd->recordset_count(); n < rs_count; ++n)
  {
    Recordset::Ref rset(mBackEnd->recordset(n));
    MResultsetViewer *rsview = [[[MResultsetViewer alloc] initWithRecordset: rset] autorelease];
    [mResultsetViews addObject: rsview];
    
    NSTabViewItem *tabViewItem= 
    [[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat:@"rset%li", rset->key()]];
    
    [tabViewItem setLabel: [NSString stringWithCPPString: rset->caption()]];
    [tabViewItem setView: [rsview view]];

    //[mLowerTabView insertTabViewItem: tabViewItem atIndex: n];
    [mLowerTabView addTabViewItem: tabViewItem];
    [rsview refresh];
  }
  
//  if ([mResultsetViews count] > 0)
//    [mLowerTabView selectFirstTabViewItem:nil];
    
  if (rs_count > 0)
    [mLowerTabView selectLastTabViewItem:nil];
}


- (void)clearResultsets
{
  for (id tabView in [[mLowerTabView tabViewItems] reverseObjectEnumerator])
  {
    if ([[tabView identifier] compare: @"rset" options:0 range:NSMakeRange(0, 4)] == NSOrderedSame)
      [mLowerTabView removeTabViewItem: tabView];
  }
  [mResultsetViews removeAllObjects];
}


static int processTaskFinish(WBSQLQueryPanel *self)
{
  //XXX[self enableCancelButton:NO];
  [self->mMessagesTable reloadData];
  [self displayResultsets];
  
  return 0;
}

static int processTaskProgress(float progress, const std::string &message, WBSQLQueryPanel *self)
{
  [self->mMessagesTable reloadData];
  
  return 0;
}

static int refreshLogTable(WBSQLQueryPanel *self)
{
  [self->mMessagesTable reloadData];
  
  return 0;
}


static int closeRecordset(long key, WBSQLQueryPanel *self)
{
  MResultsetViewer *viewer = nil;
  for (id rsview in self->mResultsetViews)
  {
    if ([rsview recordset]->key() == key)
    {
      viewer = rsview;
      break;
    }
  }

  id item = [self->mLowerTabView tabViewItemAtIndex: [self->mLowerTabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat:@"rset%li", key]]];
  
  [self->mLowerTabView removeTabViewItem: item];
  [self->mResultsetViews removeObject: viewer]; 
  
  return 0;
}


#pragma mark User actions



#define QUERY_AREA_EXPANDED_MIN_HEIGHT (100)
#define QUERY_AREA_COLLAPSED_MIN_HEIGHT (0)

#define RESULTS_AREA_EXPANDED_MIN_HEIGHT (100)
#define RESULTS_AREA_COLLAPSED_MIN_HEIGHT (28)


- (void)executeQuery:(id)sender currentStatementOnly: (bool) currentStatementOnly
{
  std::string text;
  if (currentStatementOnly)
    text= [[[self activeEditor] currentSqlStatement] UTF8String];
  else
    text= [[[self activeEditor] string] UTF8String] ? : "";
  if (text.empty())
    return;
  
  [self clearResultsets];
  
  [mLowerTabView selectTabViewItemWithIdentifier: @"messages"];
  
  //XXX[self enableCancelButton:YES];
  
  mBackEnd->exec_sql(text, false, currentStatementOnly);  
}


- (void) tabView: (NSTabView*) tabView 
draggedHandleAtOffset: (NSPoint) offset
{
  if (tabView == mLowerTabView)
  {
    NSPoint pos= [mWorkView convertPointFromBase: [[NSApp currentEvent] locationInWindow]];

    float position= pos.y - offset.y - [mWorkView dividerThickness];
    [mWorkView setPosition: position ofDividerAtIndex: 0];
  }
}


- (void) tabViewDraggerClicked: (NSTabView*) tabView 
{
  if (mLastClick > 0 && [NSDate timeIntervalSinceReferenceDate] - mLastClick < 0.3)
  {  
    if (mQueryAreaOpen && mResultsAreaOpen)
    {
      mResultsAreaOpen= NO;
      [mWorkView setPosition: NSHeight([mWorkView frame]) - RESULTS_AREA_COLLAPSED_MIN_HEIGHT
        ofDividerAtIndex: 0];
    }
    else if (mQueryAreaOpen && !mResultsAreaOpen)
    {
      mResultsAreaOpen= YES;
      mQueryAreaOpen= NO;
      [mWorkView setPosition: 0
        ofDividerAtIndex: 0];
    }
    else
    {
      mResultsAreaOpen= YES;
      mQueryAreaOpen= YES;
      
      [mWorkView setPosition: 200
        ofDividerAtIndex: 0];
    }
    mLastClick= 0;
  }
  else
    mLastClick= [NSDate timeIntervalSinceReferenceDate];
}

- (void)addEditorTabWithBackEndIndex:(int)editor_index
{
  MSQLEditorController *editor = [[[MSQLEditorController alloc] init] autorelease];
  ScintillaView *view = [[[ScintillaView alloc] initWithFrame:NSMakeRect(0, 0, 100, 100)] autorelease];
  
  [editor setView: view];
  [editor setEditorBackEnd: mBackEnd->sql_editor(editor_index)];
  
  NSTabViewItem *item = [[NSTabViewItem alloc] initWithIdentifier: editor];
  [item setView: view];
  [item setLabel: [NSString stringWithCPPString: mBackEnd->sql_editor_caption(editor_index)]];
  
  [mEditors addObject: editor];
  
  [mUpperTabView addTabViewItem:item];
  [mUpperTabView selectLastTabViewItem:nil];
  
  [[mView window] makeFirstResponder: [[editor view] content]];
}


- (void)addEditorTab
{
  int editor_index = mBackEnd->add_sql_editor();

  [self addEditorTabWithBackEndIndex: editor_index];  
}


- (MSQLEditorController*)activeEditor
{
  return [[mUpperTabView selectedTabViewItem] identifier];
}

#pragma mark Overview
- (void)updateOverviewToolbar
{
  // update overview toolbar
  bec::ToolbarItemList items = mBackEnd->live_physical_overview()->get_toolbar_items(bec::NodeId(0));
  for (bec::ToolbarItemList::const_iterator iter= items.begin(); iter != items.end(); ++iter)
  {
    switch (iter->type)
    {
      case bec::ToolbarAction:
      {
        NSButton *button = [mOverviewToolbar addButtonWithIcon: [[GRTIconCache sharedIconCache] imageForIconId: iter->icon]
                                                        target: self
                                                        action: @selector(activateOverviewToolbarItem:)
                                                           tag: 0];
        [[button cell] setRepresentedObject: [NSString stringWithCPPString: iter->command]];
        [button setToolTip: [NSString stringWithCPPString: iter->tooltip]];
        break;
      }
        
      case bec::ToolbarSeparator:
        [mOverviewToolbar addSeparator];
        break;
        
      case bec::ToolbarLabel:
        [mOverviewToolbar addLabelWithTitle: [NSString stringWithCPPString: iter->caption]];
        break;
    }
  }
}


- (void)activateOverviewToolbarItem:(id)sender
{
  std::string action = [[[sender cell] representedObject] UTF8String];
  mBackEnd->live_physical_overview()->activate_toolbar_item(bec::NodeId(0), action);
}


#pragma mark Snippets


- (void)doubleClickSnippet:(id)sender
{
  if ([snippetTable clickedRow] >= 0)
    DbSqlEditorSnippets::get_instance()->activate_node([snippetTable clickedRow]);
}


- (void)activateSnippetToolbarItem:(id)sender
{
  std::string action = [[[sender cell] representedObject] UTF8String];
  int row = [snippetTable selectedRow];
  DbSqlEditorSnippets::get_instance()->activate_toolbar_item(row, action);
}


- (void)selectSnippetCollection:(id)sender
{
  DbSqlEditorSnippets::get_instance()->select_category([[sender titleOfSelectedItem] UTF8String]);
  [snippetTable reloadData];
}

#pragma mark Public getters + setters

- (NSView*) topView;
{
  return mView;
}

- (NSString*)title
{
  return [NSString stringWithUTF8String: mBackEnd->caption().c_str()];
}


- (id)identifier
{
  return [NSString stringWithFormat:@"dbquery%p", mBackEnd.get()];
}

- (NSImage*)tabIcon
{
  return [NSImage imageNamed: @"tab.sqlquery.16x16"];
}


- (bec::UIForm*)formBE
{
  return boost::get_pointer(mBackEnd);
}


- (void)setActiveEditorTitle:(NSString*)title
{
  [[mUpperTabView selectedTabViewItem] setLabel: title];
}

- (void)updateActiveRecordsetTitle
{
  MResultsetViewer *rset = [self selectedResultset];
  if (rset)
  {
    [[mLowerTabView selectedTabViewItem] setLabel: [NSString stringWithCPPString: [rset recordset]->caption()]];
  }
}

static void refreshUIPartial(const int what, WBSQLQueryPanel *panel)
{
  switch (what)
  {
    case Db_sql_editor::RefreshSidebar:
      [panel->mSidebar refresh];
      [panel->mOverview rebuildAll];
      break;
    case Db_sql_editor::RefreshSchemaTree:
      [panel->mSidebar refreshSchemaTree];
      break;
    case Db_sql_editor::RefreshEditor:
      [[panel activeEditor] setString: [NSString stringWithCPPString: panel->mBackEnd->sql_editor()->sql()]];
      break;
    case Db_sql_editor::RefreshEditorBackend:
      panel->mBackEnd->sql_editor()->sql([[[panel activeEditor] string] UTF8String]);
      break;
    case Db_sql_editor::RefreshEditorTitle:
      [panel setActiveEditorTitle: [NSString stringWithCPPString: panel->mBackEnd->sql_editor_caption()]];
      break;
    case Db_sql_editor::RefreshSnippets:
      [panel->snippetTable reloadData];
      break;
    case Db_sql_editor::RunCurrentScript:
      [panel executeQuery: panel currentStatementOnly: false];
      break;
    case Db_sql_editor::RunCurrentStatement:
      [panel executeQuery: panel currentStatementOnly: true];
      break;
    case Db_sql_editor::RefreshRecordsetTitle:
      [panel updateActiveRecordsetTitle];
      break;
    case Db_sql_editor::RefreshOverview:
    {
      id group = [panel->mOverview itemContainerForNode: bec::NodeId(0)];
      int i = [group indexOfTabViewItem: [group selectedTabViewItem]];
      if (i != NSNotFound)
      {
        bec::NodeId node(bec::NodeId(0).append(i));
        
        for (int j= 0; j < 3; j++)
        {
          // this might not be the node that needs to be refreshed. this callback needs an extra argument
          // to tell the node to refresh
          [panel->mOverview refreshNodeChildren: bec::NodeId(node).append(j)];
        }
      }
      break;
    }
  }
}


static int insertText(const std::string &text, WBSQLQueryPanel *panel)
{
  ScintillaView *editor = [[panel activeEditor] view];
  
  [editor insertText: [NSString stringWithCPPString: text]];

  [[editor window] makeFirstResponder: [editor content]];
  
  return 0;
}

static int editorCreated(int editor_index, WBSQLQueryPanel *panel)
{
  [panel addEditorTabWithBackEndIndex: editor_index];
  return 0;
}

#pragma mark Create + destroy

- (id)initWithBE:(const Db_sql_editor::Ref&)be
{
  self= [super init];
  if (self)
  {
    [NSBundle loadNibNamed: @"WBSQLQueryPanel"
                     owner: self];
    
    mResultsetViews= [[NSMutableArray alloc] init];
    
    mBackEnd= be;
    mBackEnd->log()->refresh_ui_cb= sigc::bind(sigc::ptr_fun(refreshLogTable), self);
    
    mBackEnd->sql_editor_text_insert_signal.connect(sigc::bind(sigc::ptr_fun(insertText), self));
    
    mBackEnd->set_partial_refresh_ui_slot(sigc::bind(sigc::ptr_fun(refreshUIPartial), self));
    
    mBackEnd->exec_sql_task->progress_cb(sigc::bind(sigc::ptr_fun(processTaskProgress), self));
    mBackEnd->exec_sql_task->finish_cb(sigc::bind(sigc::ptr_fun(processTaskFinish), self));
    mBackEnd->close_recordset_ui.connect(sigc::bind(sigc::ptr_fun(closeRecordset), self));
    
    mBackEnd->sql_editor_new_ui.connect(sigc::bind(sigc::ptr_fun(editorCreated), self));
    
    mBackEnd->set_frontend_data(self);
    
    [mSplitterDelegate setTopExpandedMinHeight: QUERY_AREA_EXPANDED_MIN_HEIGHT];
    [mSplitterDelegate setTopCollapsedMinHeight: QUERY_AREA_COLLAPSED_MIN_HEIGHT];
    [mSplitterDelegate setBottomExpandedMinHeight: RESULTS_AREA_EXPANDED_MIN_HEIGHT];
    [mSplitterDelegate setBottomCollapsedMinHeight: RESULTS_AREA_COLLAPSED_MIN_HEIGHT];
  
    [mLowerTabView setDelegate: self];
    [(id)mLowerTabView createDragger];
    
    [mWorkView setDividerThickness: 0];
    [mView setDividerThickness: 3];
    
    [(MContainerView*)[mLowerTabView superview] setMinContentSize: NSMakeSize(40, RESULTS_AREA_COLLAPSED_MIN_HEIGHT+20)];
    
    [mHistoryTable setTarget: self];
    [mHistoryTable setDoubleAction: @selector(activateHistoryEntry:)];
    [mHistoryDetailsTable setTarget: self];
    [mHistoryDetailsTable setDoubleAction: @selector(activateHistoryDetailEntry:)];

    if (mBackEnd->is_physical_overview_enabled())
    {
      NSTabViewItem *item = [mLowerTabView tabViewItemAtIndex: 0];
      NSView *view;
      NSRect rect;
      
      mOverview = [[WBOverviewPanel alloc] initWithOverviewBE: mBackEnd->live_physical_overview()];
      [mOverview setNoBackground];
      [mOverview setNoHeader];
      
      view = [item view];
      rect = [view frame];
      [view addSubview: [mOverview topView]];
      rect.origin = NSZeroPoint;
      rect.size.height -= NSHeight([mOverviewToolbar frame]) + 2;
      [mOverview setFrame: rect];
      [mOverview setAutoresizingMask: NSViewHeightSizable|NSViewWidthSizable];
      
      [mOverview rebuildAll];
      
      std::string default_schema = mBackEnd->active_schema();
      if (!default_schema.empty())
      {        
        [self performSelector: @selector(activeSchemaChanged:) 
                   withObject: [NSString stringWithCPPString: default_schema]
                   afterDelay: 0.1];
      }
    }
    else
    {
      [mLowerTabView removeTabViewItem: [mLowerTabView tabViewItemAtIndex:0]];
    }

    [mSidebar setup];

    //XXX[self enableCancelButton:NO];

    mEditors = [[NSMutableArray alloc] init];
    
    mQueryAreaOpen = YES;
    mResultsAreaOpen = YES;
    
    [self updateOverviewToolbar];
    
    // update snippets list
    DbSqlEditorSnippets *snippets = DbSqlEditorSnippets::get_instance();
    [snippetDS setListModel: snippets];
    [snippetTable reloadData];
    
    [snippetTable setTarget: self];
    [snippetTable setDoubleAction: @selector(doubleClickSnippet:)];
        
    // update snippets toolbar
    bec::ToolbarItemList items = snippets->get_toolbar_items();
    for (bec::ToolbarItemList::const_iterator iter= items.begin(); iter != items.end(); ++iter)
    {
      switch (iter->type)
      {
        case bec::ToolbarAction:
        {
          NSButton *button = [mSnippetsToolbar addButtonWithIcon: [[GRTIconCache sharedIconCache] imageForIconId: iter->icon]
                                       target: self
                                       action: @selector(activateSnippetToolbarItem:)
                                          tag: 0];
          [[button cell] setRepresentedObject: [NSString stringWithCPPString: iter->command]];
          [button setToolTip: [NSString stringWithCPPString: iter->tooltip]];
          break;
        }

        case bec::ToolbarSeparator:
          [mSnippetsToolbar addSeparator];
          break;

        case bec::ToolbarLabel:
          [mSnippetsToolbar addLabelWithTitle: [NSString stringWithCPPString: iter->caption]];
          break;
      }
    }
    
    [mSnippetsToolbar addLabelWithTitle: @"Snippet Collection:"];
    [mSnippetsToolbar addSelectionPopUpWithItems: [NSArray arrayWithCPPStringVector: snippets->get_category_list()]
                                          target: self
                                          action: @selector(selectSnippetCollection:)
                                    defaultValue: nil];
    
    // realize pre-existing editors
    while ([mUpperTabView numberOfTabViewItems] < mBackEnd->sql_editor_count())
    {
      [self addEditorTabWithBackEndIndex: [mUpperTabView numberOfTabViewItems]];
    }
  }
  return self;
}


- (Db_sql_editor::Ref)backEnd
{
  return mBackEnd;
}

- (void)setRightSidebar:(BOOL)flag
{
  sidebarAtRight = flag;

  id view1 = [[mView subviews] objectAtIndex: 0];
  id view2 = [[mView subviews] objectAtIndex: 1];
  
  if (sidebarAtRight)
  {
    if (view2 != mSidebarView)
    {
      [[view1 retain] autorelease];
      [view1 removeFromSuperview];
      [mView addSubview: view1];
    }    
  }
  else
  {
    if (view1 != mSidebarView)
    {
      [[view1 retain] autorelease];
      [view1 removeFromSuperview];
      [mView addSubview: view1];
    }
  }
}

- (MResultsetViewer*)selectedResultset
{
  NSTabViewItem *selectedItem= [mLowerTabView selectedTabViewItem];

  for (id rsview in mResultsetViews)
  {
    if ([selectedItem view] == [rsview view])
      return rsview;
  }

  return nil;
}


- (void)activeSchemaChanged:(NSString*)schemaName
{
  id group = [mOverview itemContainerForNode: bec::NodeId(0)];
  int index = 0;
  for (id tab in [group tabViewItems])
  {
    if ([[tab label] isEqual: schemaName])
    {
      [group selectTabViewItemWithIdentifier: [tab identifier]];
      
      bec::NodeId node(bec::NodeId(0).append(index));
      [mOverview refreshNode: node];
      for (int j= 0; j < 3; j++)
        [mOverview refreshNodeChildren: bec::NodeId(node).append(j)];
      break;
    }
    index++;
  }
}
  
- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
  if (tabView == mUpperTabView)
  {
    MSQLEditorController *editor = [tabViewItem identifier];
    Sql_editor::Ref ed = [editor backEnd];

    for (int i= 0; i < mBackEnd->sql_editor_count(); i++)
    {
      if (mBackEnd->sql_editor(i) == ed)
      {
        mBackEnd->active_sql_editor_index(i);
        break;
      }
    }
  }
}

- (void)tabViewDidChangeNumberOfTabViewItems:(NSTabView*)tabView
{
  if (tabView == mUpperTabView)
  {
    if ([tabView numberOfTabViewItems] == 0)
      [self addEditorTab];
  }
}

- (BOOL)tabView:(NSTabView *)tabView
willCloseTabViewItem:(NSTabViewItem*)tabViewItem
{
  if (tabView == mUpperTabView)
  {
    MSQLEditorController *editor = [tabViewItem identifier];
    bool save= false;
    
    Sql_editor::Ref ed = [editor backEnd];
    int idx = -1;
    
    for (int i= 0; i < mBackEnd->sql_editor_count(); i++)
    {
      if (mBackEnd->sql_editor(i) == ed)
        idx = i;
    }
    if (idx < 0)
      return NO;

    if (mBackEnd->sql_editor_dirty(idx))
    {
      int r = NSRunAlertPanel([NSString stringWithFormat:@"Do you want to save changes made to the script \"%@\"?", [tabViewItem label]],
                              @"Your changes will be lost if you don't save them.",
                              @"Save", @"Don't Save", @"Cancel");
      if (r == NSAlertDefaultReturn)
      {
        save = true;
      }
      else if (r == NSAlertOtherReturn)
        return NO;
    }
    
    if (save && !mBackEnd->save_sql_script_file(mBackEnd->sql_editor_path(idx), idx))
      return NO;
        
    mBackEnd->remove_sql_editor(idx);
    
    return YES;
  }
  else
  {
    id ident = [tabViewItem identifier];
    if ([ident isEqual: @"history"] ||
        [ident isEqual: @"messages"])
      return NO;
    
    if ([tabViewItem view] == mOverview)
      return [mOverview willClose];
    
    MResultsetViewer *viewer = nil;
    for (id rsview in mResultsetViews)
    {
      if ([tabViewItem view] == [rsview view])
      {
        viewer = rsview;
        break;
      }
    }
    
    if (viewer)
    {
      [viewer close];
      return NO;
    }
    return YES;
  }
}

- (BOOL)willClose
{
  BOOL hasChanges = NO;
  BOOL hasEdits = NO;
  for (id rsview in mResultsetViews)
  {
    if ([rsview hasPendingChanges])
    {
      hasChanges = YES;
      break;
    }
  }
  
  for (int i = 0; i < mBackEnd->sql_editor_count(); i++)
  {
    if (mBackEnd->sql_editor_dirty(i))
    {
      hasEdits = YES;
      break;
    }
  }
  
  if (hasChanges)
  {
    if (NSRunAlertPanel(@"Close SQL Editor",
                        @"One or more resultsets have uncommited changes.\n"
                        "Would you like to Cancel and review these changes or Discard them?",
                        @"Cancel", @"Discard Changes", nil) == NSAlertDefaultReturn)
      return NO;
  }
  
  if (hasEdits)
  {
    if (NSRunAlertPanel(@"Close SQL Editor",
                        @"One or more SQL scripts have unsaved changes.\n"
                        "Would you like to Cancel and review these changes or Discard them?",
                        @"Cancel", @"Discard Changes", nil) == NSAlertDefaultReturn)
      return NO;
  }
  
  return YES;
}

- (void) dealloc
{  
  [mEditors release];
  [mResultsetViews release];
  [mOverview release];
  
  [super dealloc]; 
}


- (void)searchString:(NSString*)text
{
  [[[self activeEditor] view] findAndHighlightText:text
                                         matchCase:NO
                                         wholeWord:NO
                                          scrollTo:YES
                                              wrap:YES];
}

//--------------------------------------------------------------------------------------------------

/**
 * Executes commands sent by the main form that should be handled here.
 */
- (void) performCommand: (const std::string) command
{
  if (command == "wb.sidebarHide") // Actually means toggle.
  {
    sidebarHidden = !sidebarHidden;
    if (!sidebarHidden)
    {
      if (sidebarAtRight) 
        [mView setPosition: NSWidth([mView frame])-lastSidebarWidth ofDividerAtIndex: 0];
      else
        [mView setPosition: lastSidebarWidth ofDividerAtIndex: 0];
      NSRect frame = [mSchemaPop frame];
      frame.size.width = lastSidebarWidth - frame.origin.x - 7;
      [mSchemaPop setFrame: frame];
    }
    else
    {
      lastSidebarWidth = NSWidth([mSidebarView frame]);
      if (sidebarAtRight)
        [mView setPosition: NSWidth([mView frame]) ofDividerAtIndex: 0];
      else
        [mView setPosition: 0 ofDividerAtIndex: 0];
    }
  }
}

@end


