/* 
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "wf_webbrowser.h"

using namespace MySQL::Forms;

//----------------- WebbrowserImpl -----------------------------------------------------------------

bool WebbrowserImpl::create(::mforms::Webbrowser *self)
{
  WebbrowserImpl^ browser= gcnew WebbrowserImpl(self);
  if (browser != nullptr)
  {
    WebBrowser^ control= ViewImpl::create<WebBrowser>(self, browser);
    control->DocumentCompleted += gcnew WebBrowserDocumentCompletedEventHandler(DocumentLoaded);

    return true;
  }
  return false;
}

//-------------------------------------------------------------------------------------------------

void WebbrowserImpl::set_html(::mforms::Webbrowser *self, const std::string& code)
{
  WebbrowserImpl^ browser= (WebbrowserImpl^)ObjectImpl::FromUnmanaged(self);

  if (browser != nullptr)
    browser->set_html(code);
}

//-------------------------------------------------------------------------------------------------

void WebbrowserImpl::navigate(::mforms::Webbrowser *self, const std::string& url)
{
  WebbrowserImpl^ browser= (WebbrowserImpl^)ObjectImpl::FromUnmanaged(self);

  if (browser != nullptr)
    browser->navigate(url);
}

//-------------------------------------------------------------------------------------------------

std::string WebbrowserImpl::get_document_title(::mforms::Webbrowser *self)
{
  WebbrowserImpl^ browser= (WebbrowserImpl^)ObjectImpl::FromUnmanaged(self);

  if (browser != nullptr)
    return browser->get_document_title();

  return "";
}

//-------------------------------------------------------------------------------------------------

void WebbrowserImpl::DocumentLoaded(Object^ sender, WebBrowserDocumentCompletedEventArgs^ e)
{
  WebBrowser^ control= (WebBrowser^) sender;

  if (control->Tag != nullptr)
  {
    mforms::Webbrowser* browser= get_backend_control<mforms::Webbrowser>(control);
    if (browser != 0)
      browser->document_loaded(NativeToCppString(e->Url->AbsoluteUri));
  }
}

//----------------- WebbrowserImpl actual implementation -------------------------------------------

void WebbrowserImpl::set_html(const std::string& code)
{
  WebBrowser^ browser= get_control<WebBrowser>();
  browser->DocumentText = CppStringToNative(code);
}

//-------------------------------------------------------------------------------------------------

void WebbrowserImpl::navigate(const std::string& url)
{
  WebBrowser^ browser= get_control<WebBrowser>();
  browser->Navigate(CppStringToNative(url));
}

//-------------------------------------------------------------------------------------------------

std::string WebbrowserImpl::get_document_title()
{
  WebBrowser^ browser= get_control<WebBrowser>();
  return NativeToCppString(browser->DocumentTitle);
}

//-------------------------------------------------------------------------------------------------
